@@ -7,7 +7,7 @@ package Git::Wrapper::Plus::Refs;
 BEGIN {
   $Git::Wrapper::Plus::Refs::AUTHORITY = 'cpan:KENTNL';
 }
-$Git::Wrapper::Plus::Refs::VERSION = '0.003000';
+$Git::Wrapper::Plus::Refs::VERSION = '0.003100';
 # ABSTRACT: Work with refs
 
 use Moo qw( has );
@@ -28,7 +28,14 @@ use Moo qw( has );
 
 
 
-has git => required => 1, is => ro =>;
+has 'git' => ( required => 1, is => ro => );
+has 'support' => ( is => ro =>, lazy => 1, builder => 1 );
+
+sub _build_support {
+  my ( $self, ) = @_;
+  require Git::Wrapper::Plus::Support;
+  return Git::Wrapper::Plus::Support->new( git => $self->git );
+}
 
 
 
@@ -57,6 +64,29 @@ sub _for_each_ref {
   my ( $self, $refspec, $callback ) = @_;
 
   my $git_dir = $self->git->dir;
+
+  # git for-each-ref refs/heads/** ==
+  #     for-each-ref refs/heads/*  ==
+  #     ls-remote    refs/heads/*  + exclude refs/heads/*/*
+  #
+  # git for-each-refs refs/heads/ ==
+  #     ls-remote     refs/heads/*
+  #
+  if ( $self->support->supports_command('for-each-ref') ) {
+    ## no critic (Compatibility::PerlMinimumVersionAndWhy)
+    if ( $refspec =~ qr{\A(.*)/[*]{1,2}\z}msx ) {
+      $refspec = $1;
+    }
+    for my $line ( $self->git->for_each_ref($refspec) ) {
+      if ( $line =~ qr{ \A ([^ ]+) [^\t]+ \t ( .+ ) \z }msx ) {
+        $callback->( $1, $2 );
+        next;
+      }
+      require Carp;
+      Carp::confess( 'Regexp failed to parse a line from `git for-each-ref` :' . $line );
+    }
+    return;
+  }
   for my $line ( $self->git->ls_remote( $git_dir, $refspec ) ) {
     ## no critic (Compatibility::PerlMinimumVersionAndWhy)
     if ( $line =~ qr{ \A ([^\t]+) \t ( .+ ) \z }msx ) {
@@ -138,7 +168,7 @@ Git::Wrapper::Plus::Refs - Work with refs
 
 =head1 VERSION
 
-version 0.003000
+version 0.003100
 
 =head1 SYNOPSIS
 
